pub struct ClusterAutoScalingStrategyOptionValue {
pub on_hydration: Option<OnHydrationOptionValue>,
}Expand description
The value of the AUTO SCALING STRATEGY cluster option: the autoscaling
policy block. Extensible: future strategies are additional optional
sub-policies, so the block grows without changing existing ones. An empty
block (all sub-policies absent) disables autoscaling for the cluster, the same
as RESET (AUTO SCALING STRATEGY).
Fields
on_hydration: Option<OnHydrationOptionValue>Trait Implementations
Sourceimpl AstDisplay for ClusterAutoScalingStrategyOptionValue
impl AstDisplay for ClusterAutoScalingStrategyOptionValue
fn fmt<W: Write>(&self, f: &mut AstFormatter<W>)
fn to_ast_string_simple(&self) -> String
fn to_ast_string_stable(&self) -> String
fn to_ast_string_redacted(&self) -> String
fn to_ast_string(&self, format_mode: FormatMode) -> String
Sourceimpl Clone for ClusterAutoScalingStrategyOptionValue
impl Clone for ClusterAutoScalingStrategyOptionValue
Sourcefn clone(&self) -> ClusterAutoScalingStrategyOptionValue
fn clone(&self) -> ClusterAutoScalingStrategyOptionValue
1.0.0 (const: unstable) · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
source. Read moreSourceimpl<'de> Deserialize<'de> for ClusterAutoScalingStrategyOptionValue
impl<'de> Deserialize<'de> for ClusterAutoScalingStrategyOptionValue
Sourcefn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
fn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
Sourceimpl Ord for ClusterAutoScalingStrategyOptionValue
impl Ord for ClusterAutoScalingStrategyOptionValue
Sourcefn cmp(&self, other: &ClusterAutoScalingStrategyOptionValue) -> Ordering
fn cmp(&self, other: &ClusterAutoScalingStrategyOptionValue) -> Ordering
1.21.0 (const: unstable) · Sourcefn max(self, other: Self) -> Selfwhere
Self: Sized,
fn max(self, other: Self) -> Selfwhere
Self: Sized,
Sourceimpl PartialEq for ClusterAutoScalingStrategyOptionValue
impl PartialEq for ClusterAutoScalingStrategyOptionValue
Sourcefn eq(&self, other: &ClusterAutoScalingStrategyOptionValue) -> bool
fn eq(&self, other: &ClusterAutoScalingStrategyOptionValue) -> bool
self and other values to be equal, and is used by ==.Sourceimpl PartialOrd for ClusterAutoScalingStrategyOptionValue
impl PartialOrd for ClusterAutoScalingStrategyOptionValue
Sourceimpl<'ast, T: AstInfo> VisitMutNode<'ast, T> for ClusterAutoScalingStrategyOptionValue
impl<'ast, T: AstInfo> VisitMutNode<'ast, T> for ClusterAutoScalingStrategyOptionValue
Sourceimpl<'ast, T: AstInfo> VisitNode<'ast, T> for ClusterAutoScalingStrategyOptionValue
impl<'ast, T: AstInfo> VisitNode<'ast, T> for ClusterAutoScalingStrategyOptionValue
impl Eq for ClusterAutoScalingStrategyOptionValue
impl StructuralPartialEq for ClusterAutoScalingStrategyOptionValue