pub struct Ident(/* private fields */);Expand description
An identifier.
Implementations
Sourceimpl Ident
impl Ident
Sourcepub const MAX_LENGTH: usize = MAX_IDENTIFIER_LENGTH
pub const MAX_LENGTH: usize = MAX_IDENTIFIER_LENGTH
Maximum length of an identifier in Materialize.
Sourcepub fn new<S>(s: S) -> Result<Self, IdentError>
pub fn new<S>(s: S) -> Result<Self, IdentError>
Create a new Ident with the given value, checking our invariants.
Examples
use mz_sql_parser::ast::Ident;
let id = Ident::new("hello_world").unwrap();
assert_eq!(id.as_str(), "hello_world");
let too_long = "I am a very long identifier that is more than 255 bytes long which is the max length for idents.\
๐๐๐
๐๐ฌ๐ป๐ฎโ๐จ๐ฎ๐ฝ๐ฐ๏ธ๐๐๐๐
๐๐ฌ๐ป๐ฎโ๐จ๐ฎ๐ฝ๐ฐ๏ธ๐๐๐๐
๐๐ฌ๐ป๐ฎโ๐จ๐ฎ๐ฝ๐ฐ๏ธ๐";
assert_eq!(too_long.len(), 258);
let too_long_id = Ident::new(too_long);
assert!(too_long_id.is_err());
let invalid_name_dot = Ident::new(".");
assert!(invalid_name_dot.is_err());
let invalid_name_dot_dot = Ident::new("..");
assert!(invalid_name_dot_dot.is_err());Sourcepub fn new_lossy<S: Into<String>>(value: S) -> Self
pub fn new_lossy<S: Into<String>>(value: S) -> Self
Create a new Ident modifying the given value as necessary to meet our invariants.
Examples
use mz_sql_parser::ast::Ident;
let too_long = "๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข\
๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต\
๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด";
let id = Ident::new_lossy(too_long);
// `new_lossy` will truncate the provided string, since it's too long. Note the missing
// `๐ด` characters.
assert_eq!(id.as_str(), "๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต");Sourcepub fn new_unchecked<S: Into<String>>(value: S) -> Self
pub fn new_unchecked<S: Into<String>>(value: S) -> Self
Create a new Ident without checking any of our invariants.
NOTE: Generally you should not use this function! If youโre trying to create an
Ident from a &'static str you know is valid, use the ident! macro. For all other
use cases, see Ident::new which correctly checks our invariants.
Sourcepub fn try_generate_name<P, S, F, E>(
prefix: P,
suffix: S,
is_valid: F,
) -> Result<Self, E>
pub fn try_generate_name<P, S, F, E>( prefix: P, suffix: S, is_valid: F, ) -> Result<Self, E>
Generate a valid Ident with the provided prefix and suffix.
Examples
use mz_sql_parser::ast::{Ident, IdentError};
let good_id =
Ident::try_generate_name("hello", "_world", |_| Ok::<_, IdentError>(true)).unwrap();
assert_eq!(good_id.as_str(), "hello_world");
// Return invalid once.
let mut attempts = 0;
let one_failure = Ident::try_generate_name("hello", "_world", |_candidate| {
if attempts == 0 {
attempts += 1;
Ok::<_, IdentError>(false)
} else {
Ok(true)
}
})
.unwrap();
// We "hello_world" was invalid, so we appended "_1".
assert_eq!(one_failure.as_str(), "hello_world_1");Sourcepub fn append_lossy<S: Into<String>>(&mut self, suffix: S)
pub fn append_lossy<S: Into<String>>(&mut self, suffix: S)
Append the provided suffix, truncating self as necessary to satisfy our invariants.
Note: We soft-assert that the provided suffix is not too long, if it is, weโll
truncate it.
Examples
use mz_sql_parser::{
ident,
ast::Ident,
};
let mut id = ident!("๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต๐ต");
id.append_lossy("๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด");
// We truncated the original ident, removing all '๐ต' chars.
assert_eq!(id.as_str(), "๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด๐ด");Too long suffix
If the provided suffix is too long, weโll also truncate that.
use mz_sql_parser::{
ident,
ast::Ident,
};
let mut stem = ident!("hello___world");
let too_long_suffix = "\
๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข\
๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข\
๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข\
๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ต๐ต\
";
stem.append_lossy(too_long_suffix);
// Notice the "hello___world" stem got truncated, as did the "๐ต๐ต" characters from the suffix.
let result = "hello___wor\
๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข\
๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข\
๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข\
๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข๐ข\
";
assert_eq!(stem.as_str(), result);Sourcepub fn has_only_bare_chars(&self) -> bool
pub fn has_only_bare_chars(&self) -> bool
Reports whether the identifier matches the regex [a-z_][a-z0-9_]*,
i.e. it is composed only of characters that never require quoting.
This is the character-level half of Ident::can_be_printed_bare. It
deliberately does not consider keywords: whether a keyword-named
identifier needs quoting depends on the surrounding grammar (a
reparsing concern), not on its characters. Contexts that only need
legible, unambiguous output โ rather than a SQL round-trip โ should use
this instead (see HumanizedExplain::humanize_ident).
Sourcepub fn can_be_printed_bare(&self) -> bool
pub fn can_be_printed_bare(&self) -> bool
An identifier can be printed in bare mode if
- it matches the regex
[a-z_][a-z0-9_]*and - it is not a โreserved keyword.โ
pub fn as_str(&self) -> &str
pub fn as_keyword(&self) -> Option<Keyword>
pub fn into_string(self) -> String